"use strict";
/**
 * Certain Org Features require a translation or should be deprecated.
 * Encapsulates feature mappings and deprecated features.
 */
// P R I V A T E
Object.defineProperty(exports, "__esModule", { value: true });
// Local
const Messages = require("../messages");
const messages = Messages();
const ts_types_1 = require("@salesforce/ts-types");
const FEATURE_TYPES = {
    featureMapping: {},
    deprecatedFeatures: []
};
class ScratchOrgFeatureDeprecation {
    //Allow override for testing.
    constructor(options = FEATURE_TYPES) {
        this.featureTypes = options;
    }
    /**
     * Gets list of feature warnings that should be logged
     * @param features The requested features.
     * @returns List of string feature warnings.
     */
    getFeatureWarnings(features) {
        /* Get warning messages for deprecated features and feature mappings.*/
        const featureWarningMessages = [];
        if (features) {
            const requestedFeatures = ts_types_1.isString(features) ? features : features.join(';');
            /* If a public quantified feature is defined without a quantity, throw a warning.*/
            Object.keys(this.featureTypes.featureMapping).forEach(key => {
                if (new RegExp(`${key};|${key},|${key}$`, 'i').test(requestedFeatures)) {
                    featureWarningMessages.push(messages.getMessage('quantifiedFeatureWithoutQuantityWarning', [key, this.featureTypes.featureMapping[key]], 'signup'));
                }
            });
            /* If a deprecated feature is identified as deprecated, throw a warning.*/
            this.featureTypes.deprecatedFeatures.forEach(deprecatedFeature => {
                if (requestedFeatures.includes(deprecatedFeature)) {
                    featureWarningMessages.push(messages.getMessage('deprecatedFeatureWarning', deprecatedFeature, 'signup'));
                }
            });
        }
        return featureWarningMessages;
    }
    /**
     * Removes all deprecated features for the organization.
     * @param features List of features to filter
     * @returns feature array with proper mapping.
     */
    filterDeprecatedFeatures(features) {
        const _features = [];
        features.forEach(feature => {
            let _feature = feature;
            /* If deprecated feature is specified, remove feature from the request. */
            if (this.featureTypes.deprecatedFeatures.includes(_feature)) {
                return;
            }
            _features.push(_feature);
        });
        return _features;
    }
}
exports.ScratchOrgFeatureDeprecation = ScratchOrgFeatureDeprecation;

//# sourceMappingURL=scratchOrgFeatureDeprecation.js.map
