"use strict";
/*
 * Copyright (c) 2016, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
// Node
const fs = require("fs-extra");
const os = require("os");
const util = require("util");
const path = require("path");
// Thirdparty
const BBPromise = require("bluebird");
const _ = require("lodash");
const mkdirp = require("mkdirp");
const js2xmlparser = require('js2xmlparser');
const command_1 = require("@salesforce/command");
const OrgPrefRegistry = require("./orgPrefRegistry");
const core_1 = require("@salesforce/core");
core_1.Messages.importMessagesDirectory(__dirname);
const orgSettingsMessages = core_1.Messages.loadMessages('salesforce-alm', 'org_settings');
BBPromise.promisifyAll(fs);
/** This is the contents of the package.xml that we will use when we deploy settings to a scratch org. */
const _settingsPackageFileContents = '<?xml version="1.0" encoding="UTF-8"?>' +
    '<Package xmlns="http://soap.sforce.com/2006/04/metadata">' +
    '    <types>' +
    '%s' +
    '        <name>Settings</name>' +
    '    </types>' +
    '    <version>%s</version>' +
    '</Package>';
/**
 * Helper class for dealing with the settings that are defined in a scratch definition file.  This class knows how to extract the
 * settings from the definition, how to expand them into a MD directory and how to generate a package.xml.
 */
class SettingsGenerator {
    /** extract the settings from the scratch def file, if they are present. */
    extract(scratchDef) {
        this.settingData = _.get(scratchDef, 'settings');
        if (this.hasSettings()) {
            //TODO, this is where we will validate the settings.
            //See W-5068155
        }
    }
    /** True if we are currently tracking setting data. */
    hasSettings() {
        return !_.isEmpty(this.settingData);
    }
    /** This will copy all of the settings in the scratchOrgInfo orgPreferences mapping into the new settings structure.  It
     *  will also spit out a warning about the pending deprecation og the orgPreferences structure. */
    async migrate(scratchDef) {
        //Make sure we have old style preferences
        if (!scratchDef.orgPreferences)
            return;
        //First, let's map the old style tooling preferences into MD-API preferences
        this.settingData = {};
        const ux = await command_1.UX.create();
        function storePrefs(data, pref, prefVal) {
            const orgPrefApi = OrgPrefRegistry.whichApi(pref);
            if (util.isNullOrUndefined(orgPrefApi)) {
                ux.warn(`Unsupported org preference: ${pref}, ignored`);
                return;
            }
            const mdApiName = OrgPrefRegistry.forMdApi(pref);
            if (!_.has(data, orgPrefApi)) {
                _.set(data, orgPrefApi, {});
            }
            const apiOrgPrefs = _.get(data, orgPrefApi);
            _.set(apiOrgPrefs, mdApiName, prefVal);
        }
        if (scratchDef.orgPreferences.enabled) {
            scratchDef.orgPreferences.enabled.forEach(pref => {
                storePrefs(this.settingData, pref, true);
            });
        }
        if (scratchDef.orgPreferences.disabled) {
            scratchDef.orgPreferences.disabled.forEach(pref => {
                storePrefs(this.settingData, pref, false);
            });
        }
        // It would be nice if cli.ux.styledJSON could return a colorized JSON string instead of logging to stdout.
        const message = orgSettingsMessages.getMessage('deprecatedPrefFormat', [JSON.stringify({ orgPreferences: scratchDef.orgPreferences }, null, 4),
            JSON.stringify({ settings: this.settingData }, null, 4)]);
        ux.warn(message);
        //No longer need these
        delete scratchDef.orgPreferences;
    }
    /** Create temporary deploy directory used to upload the scratch org shape.
     * This will create the dir, generate package.xml and all of the .setting files.
     * @returns {BBPromise<string>} The temporary deployment destination directory.
     */
    async createDeployDir(apiVersion) {
        // The root of our package; use SFDX_MDAPI_TEMP_DIR if set.
        const targetDir = process.env.SFDX_MDAPI_TEMP_DIR || os.tmpdir();
        const destRoot = path.join(targetDir, 'shape');
        const settingsDir = path.join(destRoot, 'settings');
        const packageFile = path.join(destRoot, 'package.xml');
        return (fs
            .exists(destRoot)
            // Create a deploy directory
            .then(present => {
            if (present) {
                fs.removeSync(destRoot);
            }
            return mkdirp.sync(settingsDir);
        })
            // Generate all .settings files into our temporary deploy directory for each JSON setting
            .then(() => {
            const files = [];
            Object.keys(this.settingData).forEach(item => {
                const value = _.get(this.settingData, item);
                const typeName = this.cap(item);
                const fname = typeName.replace('Settings', '');
                const f = this._createSettingsFileContent(typeName, value);
                files.push(fs.writeFile(path.join(settingsDir, fname + '.settings'), f));
            });
            return BBPromise.all(files);
        })
            // Create our package.xml file in the settings root
            .then(() => {
            let memberReferences = '';
            Object.keys(this.settingData).forEach(item => {
                const typeName = this.cap(item).replace('Settings', '');
                memberReferences += '<members>' + typeName + '</members>';
            });
            return fs.writeFile(packageFile, util.format(_settingsPackageFileContents, memberReferences, apiVersion));
        })
            .then(() => destRoot));
    }
    _createSettingsFileContent(name, json) {
        if (name == `OrgPreferenceSettings`) {
            //this is a stupid format
            let res = '<?xml version="1.0" encoding="UTF-8"?>\n' +
                '<OrgPreferenceSettings xmlns="http://soap.sforce.com/2006/04/metadata">\n';
            Object.keys(json).forEach(pref => {
                res += '\t<preferences>\n';
                res += '\t\t<settingName>' + this.cap(pref) + '</settingName>\n';
                res += '\t\t<settingValue>' + _.get(json, pref) + '</settingValue>\n';
                res += '\t</preferences>\n';
            });
            res += '</OrgPreferenceSettings>';
            return res;
        }
        else {
            return js2xmlparser.parse(name, json);
        }
    }
    cap(s) {
        return s.charAt(0).toUpperCase() + s.substring(1);
    }
}
module.exports = SettingsGenerator;

//# sourceMappingURL=scratchOrgSettingsGenerator.js.map
