"use strict";
/*
 * Copyright (c) 2016, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
// Node
const util = require("util");
// Thirdparty
const BBPromise = require("bluebird");
// Local
const MdapiPackage = require("./mdapiPackage");
const metadataTypeFactory_1 = require("./metadataTypeFactory");
const forceIgnore_1 = require("./forceIgnore");
const almError = require("../core/almError");
const srcDevUtil = require("../core/srcDevUtil");
const logger = require("../core/logApi");
const Messages = require("../../lib/messages");
const messages = Messages();
/**
 * private helper to ensure the revision is a valid int.
 * @param {number} revision - the revision number
 * @returns {number} revision as a primitive integer
 * @private
 */
const _parseRevisionIntOrZero = function (revision) {
    let localRev = 0;
    if (srcDevUtil.isInt(revision)) {
        localRev = parseInt(revision);
    }
    return localRev;
};
/**
 * Helper that checks if the md item was set to obsolete in the org and returns true if so
 * @param mdApiItem
 * @returns {boolean} true - if the item is obsolete and should not be part of the md package
 * @private
 */
const _shouldExcludeFromMetadataPackage = function (mdApiItem, obsoleteNames, metadataRegistry, forceIgnore) {
    const mdFullName = mdApiItem.fullName;
    if (mdApiItem.isNameObsolete) {
        obsoleteNames.push({ fullName: mdFullName, type: mdApiItem.type });
        return true;
    }
    // check if the entity is a supported type
    if (!metadataRegistry.isSupported(mdApiItem.type)) {
        return true;
    }
    // if user wants to ignore a permissionset with fullname abc then we check if forceignore accepts abc.permissionset
    const metadataType = metadataTypeFactory_1.MetadataTypeFactory.getMetadataTypeFromMetadataName(mdApiItem.type, metadataRegistry);
    if (metadataType) {
        const filename = `${mdFullName}.${metadataType.getExt()}`;
        if (forceIgnore.denies(filename)) {
            return true;
        }
    }
    return false;
};
/**
 * Class used to derive changed org metadata.
 */
class SourceMetadataMemberRetrieveHelper {
    constructor(metadataRegistry) {
        this.metadataRegistry = metadataRegistry;
        this.scratchOrg = metadataRegistry.org;
        this.force = this.scratchOrg.force;
        this.forceIgnore = new forceIgnore_1.ForceIgnore();
        this.logger = logger.child('SourceMetadataMemberRetrieveHelper');
    }
    /**
     * returns the last edit revision count on the scratch org.
     * @returns {BBPromise}
     */
    // TODO: We should be able to get this when we are querying the SourceMember initially.
    getLatestSourceRevisionCount() {
        const query = 'SELECT MAX(RevisionNum) maxRev from SourceMember';
        return this.force.toolingQuery(this.scratchOrg, query).then(result => {
            if (!util.isNullOrUndefined(result) && result.records.length > 0) {
                return BBPromise.resolve(result.records[0].maxRev);
            }
            return BBPromise.reject(almError('invalidResponseFromQuery', [query]));
        });
    }
    shouldAddMember(mdApiItem, obsoleteNames) {
        return mdApiItem !== null &&
            !_shouldExcludeFromMetadataPackage.call(this, mdApiItem, obsoleteNames, this.metadataRegistry, this.forceIgnore);
    }
    /**
     * gets all source metadata revisions from the server from <fromRevision>.
     * @param {number} fromRevision - get all changes after this revision number
     * @returns
     * "Package": {
     *   "$": {
     *     "xmlns": "http://soap.sforce.com/2006/04/metadata"
     *   },
     *   "types": [
     *     {
     *       "name": "ApexClass",
     *       "members": [...]
     *     },
     *     ...
     *   ],
     *   "version": 38
     *}
     */
    getRevisionsAsPackage(fromRevision, obsoleteNames) {
        const localFromRevision = _parseRevisionIntOrZero(fromRevision);
        const mdPackage = new MdapiPackage();
        const whereClause = { RevisionNum: { $gt: localFromRevision } };
        return this.force
            .toolingFind(this.scratchOrg, 'SourceMember', whereClause, [
            'MemberIdOrName',
            'RevisionNum',
            'MemberType',
            'MemberName',
            'IsNameObsolete'
        ])
            .then(results => {
            const parsePromises = results.map(sourceMember => {
                const memberType = sourceMember.MemberType;
                const memberName = sourceMember.MemberName;
                if (util.isNullOrUndefined(memberType) || util.isNullOrUndefined(memberName)) {
                    throw almError('fullNameIsRequired');
                }
                const metadataType = metadataTypeFactory_1.MetadataTypeFactory.getMetadataTypeFromMetadataName(memberType, this.metadataRegistry);
                if (metadataType) {
                    return metadataType.parseSourceMemberForMetadataRetrieve(sourceMember.MemberName, sourceMember.MemberType, sourceMember.IsNameObsolete);
                }
                else {
                    this.logger.log(messages.getMessage('metadataTypeNotSupported', memberType));
                    return null;
                }
            });
            return BBPromise.all(parsePromises);
        })
            .then(promisedResults => {
            promisedResults.forEach(mdApiItem => {
                if (this.shouldAddMember(mdApiItem, obsoleteNames)) {
                    mdPackage.addMember(mdApiItem.fullName, mdApiItem.type);
                }
            });
            return mdPackage;
        });
    }
}
module.exports = SourceMetadataMemberRetrieveHelper;

//# sourceMappingURL=sourceMetadataMemberRetrieveHelper.js.map
